JSON = require "lua/JSON.lua"

-- paths
local IMG_DIR = "Story/"
local IMG_DIR_ETC = "Story/etc/"
local IMG_DIR_SOUND = "Story/sound/"
local IMG_DIR_LOCAL = "/storage/emulated/0/Pictures/NightmareForeign-testlua/res/"

-- debug Auto touch mode(1:on 0:off)
local AUTO_TOUCH_MODE = 0

-- local resource mode(1:on 0:off)
local LOCAL_RESOURCE_MODE = 0

-- scene child id --
local LAYER_MSG = 1
local LAYER_NAME = 2
local NODE_STORY = 3
local LAYER_FADE = 4
local IMG_SHUTTER = 5
local LAYER_IF = 6

local TOP_BK_FRAME = 500
local BOTTOM_BK_FRAME = 501

-- message child id --
local IMG_MSG = 1
local IMG_NAR = 2
local IMG_STAR = 6

-- name child id --
local IMG_NAME = 1
local LABEL_NAME = 2

-- story child id --
local IMG_IN_NODE = 1

-- animation frame and speed --
local STAR_ANIMATION_FRAME = 4
local STAR_ANIMATION_INTERVAL = 0.1
local FADE_ANIMATION_INTERVAL = 1

-- base sizes of parts --
local STORY_SCENE_WIDTH = 640.0
local STORY_SCENE_HEIGHT = 960.0

local TOP_BK_FRAME_HEIGHT = 400.0
local BOTTOM_BK_FRAME_HEIGHT = 400.0

local MESSAGE_LAYER_OFFSET_X = 320.0
local MESSAGE_LAYER_OFFSET_Y = 90.0

local NAME_LAYER_OFFSET_X = 170.0
local NAME_LAYER_OFFSET_Y = 210.0

local MSG_BODY_POS_X = -260
local MSG_BODY_POS_Y = 60

local LABEL_NAME_POS_X = -120.0
local LABEL_NAME_POS_Y = 2.0

local ICON_POS_X = 210.0
local ICON_POS_Y = -40.0

local IF_SPACE_HEIGHT = 450.0
local IF_SELECT_POS_X = 100.0
local IF_SELECT_POS_Y = 54.0
local IF_PADDING_WIDTH = 0.8
local IF_PADDING_HEIGHT = 0.8

local AD_AREA_HEIGHT = 100.0

-- font color --
local MSG_BODY_C_R = 64
local MSG_BODY_C_G = 64
local MSG_BODY_C_B = 64

local LABEL_NAME_C_R = 0
local LABEL_NAME_C_G = 0
local LABEL_NAME_C_B = 0

local LABEL_SELECT_C_R = 255
local LABEL_SELECT_C_G = 255
local LABEL_SELECT_C_B = 255

-- image file name --
local IMG_NAME_MES_BODY = "dialog.png"
local IMG_NAME_NARRATION = "dialog.png"
local IMG_NAME_ICON = "icon.png"
local IMG_NAME_NAME = "name.png"
local IMG_NAME_SHUTTER = "shutter.png"
local IMG_NAME_SELECT = "select.png"
local IMG_NAME_SELECTED = "selected.png"

-- font size --
local LABEL_NAME_SIZE = 22
local LABEL_SELECT_SIZE = 18

-- position argument --
local CHARA_POS_DIR_LEFT = -1
local CHARA_POS_DIR_CENTER = 0
local CHARA_POS_DIR_RIGHT = 1

-- character fix position
local CHARA_LEFT = -120
local CHARA_CENTER = 0
local CHARA_RIGHT = 120

-- global status --
local res2screen_rate = 1.0
local gcclayer = nil

local user_name = ""
local last_name = ""
local first_name = ""

local select_meta = {}
local selected_label = "label1"

local scene_skip_no = 0
local key_wait_flag = false
local message_animation = false

local scripts = nil
local scripts_cursol = 1

local sound_playing = false

local res_data = {}
local node_data = {}
local img_data = {}

-- flag --
local story_flag = ""
local flag_point = 100

-- util function --
local function lualog(...)
    print(string.format(...))
end

local function screen_ccp(x,y)
	return ccp( x * res2screen_rate, y * res2screen_rate )
end

local function makePath( imgPath )
	if LOCAL_RESOURCE_MODE == 1 then
		-- body
	    local filePath = IMG_DIR_LOCAL
	    local path = filePath .. imgPath
	    lualog("makePath = " .. path)
	    return path
    else
		-- body
	    local fileutils = CCFileUtils:sharedFileUtils()
	    local filePath = fileutils:getWritablePath()
	    local path = filePath .. imgPath
	    lualog("makePath = " .. path)
	    return path
    end
end

-- event function --
local function onTouchBegan(x, y)
    lualog("onTouchBegan: %0.2f, %0.2f", x, y)
    
    if key_wait_flag then
    	lualog("resume")
    	key_wait_flag = false
    	nextScript()
    elseif message_animation then
    	MessageAnimationSkip(LAYER_MSG);
    end
    
    return true
end

local function onTouchMoved(x, y)
	lualog("onTouchMoved: %0.2f, %0.2f", x, y)
	return true
end

local function onTouchEnded(x, y)
	lualog("onTouchEnded: %0.2f, %0.2f", x, y)
	return true
end

local function onTouch(eventType, x, y)
	lualog("onTouch: %s %0.2f, %0.2f", eventType, x, y)
	if eventType == "began" then
		return onTouchBegan(x, y)
	elseif eventType == "moved" then
		return onTouchMoved(x, y)
	else
		return onTouchEnded(x, y)
	end
end

-- ソートされたハッシュを取得する --
local function orderPairs(tab)
    local sorted = {}
    for key in pairs(tab) do
        table.insert(sorted,key)
    end
    table.sort(sorted)
    local i=0
    return function()
        i = i + 1
        if i > #sorted then
            return nil,nil
        else
            local key=sorted[i]
            return key,tab[key]
        end
    end
end

-- tagを指定したnodeを作成し親nodeに追加する --
local function addNodes(parent,children)

	local zorder = 1
	for node_id, w_children in orderPairs(children) do

		local n = CCNode:create()
		parent:addChild(n,zorder)
		node_data[node_id] = n
		
		if w_children ~= nil then
			addNodes(n,w_children)
		end
		zorder = zorder + 1
	end
end

-- 指定されたnodeのイメージを削除する --
local function removeImage(n)
	local children = n:getChildren()
	if children ~= nil then
		local img = nil
		for i = 0 , children:count() - 1 do
			local child = children:objectAtIndex(i)
			if child:getTag() == IMG_IN_NODE then
				img = child
			else
				removeImage(child)
			end
		end
		if img ~= nil then
			img:removeFromParentAndCleanup(true)
		end
	end
end

-- 指定されたnodeの全ての子孫のイメージを取得する --
local function enumImages(n,images)
	local children = n:getChildren()
	if children ~= nil then
		for i = 0 , children:count() - 1 do
			local child = children:objectAtIndex(i)
			if child:getTag() == IMG_IN_NODE then
				table.insert(images,child)
			else
				images = enumImages(child,images)
			end
		end
	end
	return images
end

-- setup function --
function storySetupEx(cclayer, last_name_data, first_name_data, love_point, flag)

	lualog( 'call storySetupEx( ' .. love_point .. ', ' .. flag .. ' )' )

    -- cocos object --
    gcclayer = cclayer;
    
    -- screen size --
    local pDirector = CCDirector:sharedDirector()
    res2screen_rate = pDirector:getWinSize().width / STORY_SCENE_WIDTH
    pDirector:setContentScaleFactor(STORY_SCENE_WIDTH / pDirector:getWinSize().width)
	
	-- regist event --
	gcclayer:setTouchEnabled(true)
	gcclayer:registerScriptTouchHandler(onTouch)
	
	-- momery setting --
	collectgarbage("setpause", 100)
	collectgarbage("setstepmul", 5000)
	
	-- user data --
	last_name = last_name_data
	first_name = first_name_data
	flag_point = tonumber(love_point)
	story_flag = flag

    return STORY_SCENE_WIDTH
end

function setupResource(json)

	scene_skip_no = 0
	selected_label = "label1"
	res_data = JSON:decode(json)
	
	local pDirector = CCDirector:sharedDirector()
	local win_size = pDirector:getWinSize()
	local device_height = win_size.height / res2screen_rate

	-- 広告領域とストーリー再生領域の高さの合計
	local total_height = STORY_SCENE_HEIGHT + AD_AREA_HEIGHT
	if total_height > device_height then
		total_height = device_height
	end

	-- ストーリー再生領域のサイズ --
	local frame_size = ccp( STORY_SCENE_WIDTH, total_height - AD_AREA_HEIGHT )

	-- ストーリー再生領域の左下の位置 --
	local frame_pos = ccp( 0, device_height - total_height )
	
	--========== load image ==========--
	for i = 1, #res_data["images"] do
		local l = CCSprite:create(makePath(IMG_DIR .. res_data["images"][i]))
		if l == nil then
			lualog('no image ' .. res_data["images"][i])
			l =	CCSprite:create("dummy.png")
		end

		l:retain()
		img_data[tostring(i)] = l

		local sz = l:getContentSize()
		if (sz.width > STORY_SCENE_WIDTH * res2screen_rate + 1) or (sz.width < STORY_SCENE_WIDTH * res2screen_rate - 1) then
			l:setScaleX( STORY_SCENE_WIDTH * res2screen_rate / sz.width )
			l:setScaleY( STORY_SCENE_WIDTH * res2screen_rate / sz.width )
		end
	end

	--========== story node ==========--
	local story = CCNode:create()
	story:setTag(NODE_STORY)
	gcclayer:addChild(story,1)
	addNodes(story,res_data["nodes"])

	-- position --
	local clipping_ofs = frame_size.y - STORY_SCENE_HEIGHT
	local scene_ofs = screen_ccp( STORY_SCENE_WIDTH / 2, clipping_ofs + STORY_SCENE_HEIGHT / 2 )
	story:setPosition( ccp(frame_pos.x + scene_ofs.x, frame_pos.y + scene_ofs.y) )
	
	--========== message layer ==========--
	local layer_msg = CCSprite:create("dummy.png")
	layer_msg:setVisible(false)
	layer_msg:setTag(LAYER_MSG)
	gcclayer:addChild(layer_msg, 70)

	local l = CCSprite:create(makePath(IMG_DIR_ETC .. IMG_NAME_MES_BODY))
	l:setTag(IMG_MSG)
	layer_msg:addChild(l, 0)

	l = CCSprite:create(makePath(IMG_DIR_ETC .. IMG_NAME_NARRATION))
	l:setTag(IMG_NAR)
	layer_msg:addChild(l, 0)

	local layer_icon = CCSprite:create(makePath(IMG_DIR_ETC .. IMG_NAME_ICON))
	if layer_icon ~= nil then
		local frames = CCArray:create()

		for i=1, STAR_ANIMATION_FRAME, 1 do
		    local frame =  CCSpriteFrame:create(makePath(string.format(IMG_DIR_ETC .. IMG_NAME_ICON, i)), layer_icon:getTextureRect())
			local offset = frame:getOffset()
			offset.y = offset.y + i - 1
			frame:setOffset( offset )
		    frames:addObject( frame )
		end

		local animation_frames = CCAnimation:createWithSpriteFrames(frames, STAR_ANIMATION_INTERVAL)
		local animate_track = CCAnimate:create(animation_frames)
		local animate_repeator = CCRepeatForever:create(animate_track)
		
		layer_icon:setPosition( screen_ccp( ICON_POS_X, ICON_POS_Y ) )
		layer_icon:setTag(IMG_STAR)
		layer_icon:runAction(animate_repeator)
		
		layer_msg:addChild(layer_icon, 1)
	end

	-- position --
	local msg_ofs = screen_ccp( MESSAGE_LAYER_OFFSET_X, MESSAGE_LAYER_OFFSET_Y )
	layer_msg:setPosition( ccp(msg_ofs.x + frame_pos.x, msg_ofs.y + frame_pos.y) )

	--========== name layer ==========--
	local layer_name = CCSprite:create("dummy.png");
	layer_name:setVisible(false)
	layer_name:setTag(LAYER_NAME)
	gcclayer:addChild(layer_name, 70)

	l = CCSprite:create(makePath(IMG_DIR_ETC .. IMG_NAME_NAME))
	local name_size = l:boundingBox()
	l:setTag(IMG_NAME)
	layer_name:addChild(l)

	l = CCLabelTTF:create("", "arial", LABEL_NAME_SIZE * res2screen_rate);
	l:setAnchorPoint(ccp(0.0, 0.5))
	l:setPosition(screen_ccp(LABEL_NAME_POS_X, LABEL_NAME_POS_Y))
	l:setColor(ccc3(LABEL_NAME_C_R,LABEL_NAME_C_G,LABEL_NAME_C_B))
	l:setTag(LABEL_NAME)
	layer_name:addChild(l)

	-- position --
	local name_ofs = screen_ccp( NAME_LAYER_OFFSET_X, NAME_LAYER_OFFSET_Y )
	layer_name:setPosition( ccp(name_ofs.x + frame_pos.x, name_ofs.y + frame_pos.y) )
    
	--========== if layer ==========--
	local layer_if = CCSprite:create("dummy.png");
	layer_if:setVisible(false)
	layer_if:setTag(LAYER_IF)
	gcclayer:addChild(layer_if, 70)
	
	-- position --
	local if_ofs = screen_ccp( STORY_SCENE_WIDTH / 2, frame_size.y / 2 )
	layer_if:setPosition(ccp(if_ofs.x + frame_pos.x, if_ofs.y + frame_pos.y))

	--========== fade layer ==========--
    local layer_fade = CCLayerColor:create( ccc4( 0, 0, 0, 255 ), win_size.width, win_size.height )
	layer_fade:setTag(LAYER_FADE)
	layer_fade:setOpacity(0)
    gcclayer:addChild(layer_fade, 50)

	--========== shutter layer ==========--
	local layer_shutter = CCSprite:create ( makePath(IMG_DIR_ETC .. IMG_NAME_SHUTTER) )
	layer_shutter:setTag(IMG_SHUTTER)
	layer_shutter:setOpacity(0)
	gcclayer:addChild(layer_shutter, 50)
    
	--========== top frame layer ==========--
	top_bk_frame = CCLayerColor:create( ccc4( 0, 0, 0, 255 ), win_size.width, TOP_BK_FRAME_HEIGHT * res2screen_rate )
	top_bk_frame:setTag(TOP_BK_FRAME)
	gcclayer:addChild(top_bk_frame, 100)
	
	-- position --
	local top_ofs = screen_ccp( 0, frame_size.y )
	top_bk_frame:setPosition( ccp( top_ofs.x + frame_pos.x, top_ofs.y + frame_pos.y ) )

	--========== bottom frame layer ==========--
	bottom_bk_frame = CCLayerColor:create( ccc4( 0, 0, 0, 255 ), win_size.width, BOTTOM_BK_FRAME_HEIGHT * res2screen_rate )
	bottom_bk_frame:setTag(BOTTOM_BK_FRAME)
	gcclayer:addChild(bottom_bk_frame, 100)
	
	-- position --
	local bottom_ofs = screen_ccp(0, - BOTTOM_BK_FRAME_HEIGHT)
	bottom_bk_frame:setPosition( ccp( bottom_ofs.x + frame_pos.x, bottom_ofs.y + frame_pos.y ) )
end

-- script function --
function startScript(script_json)
	if AUTO_TOUCH_MODE == 1 then
		StartAutoTouch()
	end

	scripts = JSON:decode(script_json)
	scripts_cursol = 0;
	select_meta = {}
	selected_label = "label1"
	key_wait_flag = false
	message_animation = false
	
	nextScript()
end

function nextScript()
	scripts_cursol = scripts_cursol + 1

	local script_str = scripts[scripts_cursol];
	script_str = string.gsub(script_str, "\n", "\\n");
	local command_str = string.sub(script_str, 1, 3)

	lualog(string.gsub(script_str,"%%","%%%%"))
	local cmd = "commands:" .. script_str
	assert(loadstring(cmd))()
end

function messageAnimationEnd()
	lualog('messageAnimationEnd')
	
	if message_animation then
		gcclayer:getChildByTag(LAYER_MSG):getChildByTag(IMG_STAR):setVisible(true);
		gcclayer:runAction( CCCallFunc:create(nextScript) )
	end
	
	message_animation = false
end

-- command function --
commands = {}

-- 名前ウインドウに表示する内容を入れます --
function commands:NAM(str)
	lualog('NAM:called: ' .. str)
    
    if str == nil then
		str = ""
	end

	local layer = gcclayer:getChildByTag(LAYER_NAME)

	if str == "$OWNER" then
	    layer:setVisible(false)
	    nextScript()
		return
	end 

    local show_str = string.gsub(str, "$LASTNAME", last_name);
    show_str = string.gsub(show_str, "$FIRSTNAME", first_name);
    show_str = string.gsub(show_str, "　", " ");
    show_str = string.gsub(show_str, "  ", " ");
	
	local label = layer:getChildByTag(LABEL_NAME)

    label:setString(show_str)
    layer:setVisible(true)
    
	nextScript()
end

-- 会話ウインドウに表示する内容を入れます --
function commands:MES(line1, line2, line3)
	lualog('MES:called')
	
	local layer = gcclayer:getChildByTag(LAYER_MSG)
	layer:setVisible(true)
	
	local img = layer:getChildByTag(IMG_MSG)
	img:setVisible(true)
	
	img = layer:getChildByTag(IMG_NAR)
	img:setVisible(false)
	
	if line1 == nil then
		line1 = ""
	end
	
    local show_str = string.gsub(line1, "$USER", user_name);
    show_str = string.gsub(show_str, "$LASTNAME", last_name);
    show_str = string.gsub(show_str, "$FIRSTNAME", first_name);
	
	layer:getChildByTag(IMG_STAR):setVisible(false);
	-- MessageAnimationStart(LAYER_MSG, show_str)
	MessageAnimationStart(LAYER_MSG, show_str, MSG_BODY_POS_X, MSG_BODY_POS_Y, MSG_BODY_C_R, MSG_BODY_C_G, MSG_BODY_C_B)
	message_animation = true
end

-- ナレーションウインドウに表示する内容を入れます --
function commands:NAR(line1)
    commands:NAR(line1, "", "")
end

function commands:NAR(line1, line2, line3)
	lualog('NAR:called')
    
    gcclayer:getChildByTag(LAYER_NAME):setVisible(false)
    
	local layer = gcclayer:getChildByTag(LAYER_MSG)
	layer:setVisible(true)
	
	local img = layer:getChildByTag(IMG_NAR)
	img:setVisible(true)
	
	img = layer:getChildByTag(IMG_MSG)
	img:setVisible(false)
	
	if line1 == nil then
		line1 = ""
	end
	
    local show_str = string.gsub(line1, "$USER", user_name);
    show_str = string.gsub(show_str, "$LASTNAME", last_name);
    show_str = string.gsub(show_str, "$FIRSTNAME", first_name);
	
	layer:getChildByTag(IMG_STAR):setVisible(false);
	-- MessageAnimationStart(LAYER_MSG, show_str)
	MessageAnimationStart(LAYER_MSG, show_str, MSG_BODY_POS_X, MSG_BODY_POS_Y, MSG_BODY_C_R, MSG_BODY_C_G, MSG_BODY_C_B)
	message_animation = true
end

-- 音の再生をします --
function commands:PLA(id)
	if sound_playing then
		StopSound()
		sound_playing = false
	end
	if res_data["sound"..id] ~= "" and res_data["sound"..id] ~= 0 and res_data["sound"..id] ~= nil then
		local bgmPath = makePath(IMG_DIR_SOUND .. res_data["sound"..id])
		PlaySound(bgmPath)
		sound_playing = true
	end
	gcclayer:runAction( CCCallFunc:create(nextScript) )
end

-- 音の停止をします --
function commands:STO(id)
	if sound_playing then
		StopSound()
		sound_playing = false
	end
	gcclayer:runAction( CCCallFunc:create(nextScript) )
end

-- 画像を指定nodeにセットする
function commands:SET(node_id,img_id)
	lualog('SET:called')
	
	local n = node_data[node_id]
	removeImage(n)

	local img = img_data[img_id]
	
	-- イメージがほかのレイヤーで既に使用されている場合
	if img:getParent() ~= nil then
		lualog('copy image ' .. res_data["images"][tonumber(img_id)])
		local copy = CCSprite:createWithTexture(img:getTexture())
		copy:setScaleX( img:getScaleX() )
		copy:setScaleY( img:getScaleY() )
		img = copy
	end

	n:addChild(img,0,IMG_IN_NODE)
	
	-- 透過度を設定する(ルート直下の場合は非透過、それ以外の場合は親の透過度を引き継ぐ)
	local p = n:getParent()
	if p:getTag() == NODE_STORY then
		img:setOpacity(255)
	else
		local w_img = p:getChildByTag(IMG_IN_NODE)
		if w_img ~= nil then
			img:setOpacity(w_img:getOpacity())
		else
			img:setOpacity(255)
		end
	end

	-- 表示する(ルート直下以外は親が表示されていなければ表示されない)
	n:setVisible(true)
	
	gcclayer:runAction( CCCallFunc:create(nextScript) )
end

-- 画面全体のFadeIn(指定された色のfade用nodeを徐々に透過にする) --
function commands:FIN(red, green, blue)
	lualog('FIN:called')
	
	local fade = gcclayer:getChildByTag(LAYER_FADE)
	fade:setColor(ccc3(tonumber(red),tonumber(green),tonumber(blue)))
	
	local tracks = CCArray:create()
	
	opacity = fade:getOpacity()
	lualog('FIN:opacity:' .. opacity)
	
	if opacity ~= 0 then
	    tracks:addObject( CCFadeOut:create( FADE_ANIMATION_INTERVAL ) )
	end
	
	tracks:addObject( CCCallFunc:create(nextScript) )
	fade:runAction( CCSequence:create(tracks) )
end

-- 画面全体のFadeOut(指定された色のfade用nodeを徐々に非透過にする) --
function commands:FOU(red, green, blue)
	lualog('FOU:called')
	
	local fade = gcclayer:getChildByTag(LAYER_FADE)
	fade:setColor(ccc3(tonumber(red),tonumber(green),tonumber(blue)))
	local tracks = CCArray:create()
	
	opacity = fade:getOpacity()
	lualog('FOU:opacity:' .. opacity)
	
	if opacity ~= 255 then
	    tracks:addObject( CCFadeIn:create( FADE_ANIMATION_INTERVAL ) )
	end
	
	tracks:addObject( CCCallFunc:create(nextScript) )
	fade:runAction( CCSequence:create(tracks) )
end

-- 指定nodeのFadeIn --
function commands:NFI(node_id)
	lualog('NFI:called')
	
	local n = node_data[node_id]
	local tracks = CCArray:create()
	local fade_action = CCArray:create()
	
	images = enumImages(n,{})
	for k, v in pairs(images) do
		if v:getOpacity() ~= 255 then
			fade_action:addObject( CCTargetedAction:create(v,CCFadeIn:create( FADE_ANIMATION_INTERVAL )) )
		end
	end	

	if fade_action:count() > 0 then
		tracks:addObject(CCSpawn:create(fade_action))
	end
	
	tracks:addObject(CCCallFunc:create(nextScript))
	n:runAction( CCSequence:create(tracks) )
end

-- 指定nodeのFadeOut --
function commands:NFO(node_id)
	lualog('NFO:called')
	
	local n = node_data[node_id]
	local tracks = CCArray:create()
	local fade_action = CCArray:create()
	
	images = enumImages(n,{})
	for k, v in pairs(images) do
		if v:getOpacity() ~= 0 then
			fade_action:addObject( CCTargetedAction:create(v,CCFadeOut:create( FADE_ANIMATION_INTERVAL )) )
		end
	end	

	if fade_action:count() > 0 then
		tracks:addObject(CCSpawn:create(fade_action))
	end
	
	tracks:addObject(CCCallFunc:create(nextScript))
	n:runAction( CCSequence:create(tracks) )
end

-- 指定nodeの表示・非表示(同時に透過度も設定する) --
function commands:VIS(node_id,visible)
	lualog('VIS:called')
	
	visible = tonumber(visible)

	local n = node_data[node_id]
    n:setVisible( (visible == 1) )

	-- 表示の場合は非透過、非表示の場合は透過に設定する
	images = enumImages(n,{})
	for k, v in pairs(images) do
		v:setOpacity((visible == 1) and 255 or 0)
	end	
    
    gcclayer:runAction( CCCallFunc:create(nextScript) )
end

-- テキストボックス枠の表示/非表示を切り替える --
function commands:TXT(visible)
	lualog('TXT:called')
	
	visible = tonumber(visible)

	local nan = gcclayer:getChildByTag(LAYER_NAME)
	local msg = gcclayer:getChildByTag(LAYER_MSG)
		
	nan:setVisible((visible == 1))
	msg:setVisible((visible == 1))

    gcclayer:runAction( CCCallFunc:create(nextScript) )
end

-- 指定されたnodeを指定された位置に移動する --
function commands:MOV(node_id,pos,time)
	lualog('MOV:called')
	
	pos = tonumber(pos)
	time = tonumber(time)

	local n = node_data[node_id]
	local moved_pos = nil
    
	if pos == CHARA_POS_DIR_LEFT then
		moved_pos = CHARA_LEFT * res2screen_rate
	end
    
    if pos == CHARA_POS_DIR_CENTER then
        moved_pos = CHARA_CENTER * res2screen_rate
    end
    
    if pos == CHARA_POS_DIR_RIGHT then
        moved_pos = CHARA_RIGHT * res2screen_rate
    end

    if time == 0 then
        n:setPositionX(moved_pos)
        gcclayer:runAction( CCCallFunc:create(nextScript) )
    else
        local tracks = CCArray:create()
        tracks:addObject( CCMoveTo:create( time/10, ccp(moved_pos, 0) ) )
        tracks:addObject( CCCallFunc:create(nextScript) )
        n:runAction( CCSequence:create(tracks) )
    end
end

-- キー待ち処理 --
function commands:KWA()
	lualog('KWA:called')
	
	key_wait_flag = true
end

-- 終了 --
function commands:EEN( next_label )
	lualog("end: next_label=" .. next_label .. " selected_label=" .. selected_label )
	
	if next_label ~= nil and next_label ~= "NULL" then
		selected_label = next_label
	end

	local l = nil
	
	l = gcclayer:getChildByTag(NODE_STORY)
	if l ~= nil then
		l:removeAllChildrenWithCleanup(true)
		l:removeFromParentAndCleanup(true)
	end
	node_data = {}

	for i = 1, #img_data do
		img_data[i]:release()
	end
	img_data = {}
	
	l = gcclayer:getChildByTag(LAYER_MSG)
	if l ~= nil then
		l:removeAllChildrenWithCleanup(true)
		l:removeFromParentAndCleanup(true)
	end
	
	l = gcclayer:getChildByTag(LAYER_NAME)
	if l ~= nil then
		l:removeAllChildrenWithCleanup(true)
		l:removeFromParentAndCleanup(true)
	end

	if gcclayer:getChildByTag(LAYER_FADE) ~= nil then
		gcclayer:removeChildByTag(LAYER_FADE, true)
	end
	
	if gcclayer:getChildByTag(IMG_SHUTTER) ~= nil then
		gcclayer:removeChildByTag(IMG_SHUTTER, true)
	end
	
	l = gcclayer:getChildByTag(LAYER_IF)
	if l ~= nil then
		l:removeAllChildrenWithCleanup(true)
		l:removeFromParentAndCleanup(true)
	end

	if gcclayer:getChildByTag(TOP_BK_FRAME) ~= nil then
		gcclayer:removeChildByTag(TOP_BK_FRAME, true)
	end

	if gcclayer:getChildByTag(BOTTOM_BK_FRAME) ~= nil then
		gcclayer:removeChildByTag(BOTTOM_BK_FRAME, true)
	end

	if AUTO_TOUCH_MODE == 1 then
		StopAutoTouch()
	end

	EndScript(selected_label, scene_skip_no, story_flag)
end

--============ if attribute ============--
function btnIfCallback(btnId)
	if select_meta[btnId].attr and selected_label == "label1" then
		selected_label = select_meta[btnId].label
		lualog("btnIfCallback:" .. selected_label)
		nextScript()
	end
end

-- IF:属性 --
function commands:IFA(...)
	lualog('IFA:called')
	
	local args = { ... }
	local layer = gcclayer:getChildByTag(LAYER_IF)
	layer:setVisible(false)

	for k, v in pairs(args) do
		select_meta[k] = {}
		select_meta[k].attr = (tonumber(v) == 1)
	end
		
	nextScript()
end

-- IF:選択肢名 --
function commands:IFN(...)
	lualog('IFN:called')
	
	local args = { ... }
	local layer = gcclayer:getChildByTag(LAYER_IF)
	
	local height = IF_SPACE_HEIGHT
	local interval = height / (#args + 1)
	local harf_height = height / 2
	local getIfButtonCallback = function(v)
		local id = v
		return function()
			btnIfCallback(id);
		end
	end

	for k, v in pairs(args) do
		-- if button --
		local pos_y = -(interval * k - harf_height) * res2screen_rate
		local img = CCMenuItemImage:create(makePath(IMG_DIR_ETC .. IMG_NAME_SELECT),makePath(IMG_DIR_ETC .. IMG_NAME_SELECTED))
		img:setOpacity(select_meta[k].attr and 255 or 120)
		img:registerScriptTapHandler(getIfButtonCallback(k))

		local sz = img:getContentSize()
		sz.width = sz.width * IF_PADDING_WIDTH
		sz.height = sz.height * IF_PADDING_HEIGHT
		
		local label = CCLabelTTF:create("", "arial", LABEL_SELECT_SIZE * res2screen_rate, sz, kCCTextAlignmentLeft, kCCVerticalTextAlignmentCenter);
		label:setColor(ccc3(LABEL_SELECT_C_R,LABEL_SELECT_C_G,LABEL_SELECT_C_B))
		label:setOpacity(select_meta[k].attr and 255 or 120)
		label:setPosition(screen_ccp(IF_SELECT_POS_X, IF_SELECT_POS_Y))
		label:setAnchorPoint(ccp(0.0, 0.5))
		label:setString(v)
		img:addChild(label)
		
		local menu = CCMenu:createWithItem(img)
		menu:setPosition(ccp(0, pos_y))
		menu:setTag(k)
		layer:addChild(menu)
		
		select_meta[k].name = v
	end

	nextScript()
end

-- IF:選択肢説明 --
function commands:IFE(...)
	lualog('IFE:called')
	
	nextScript()
end

-- IF:選択肢パラメータ(遷移先ID) --
function commands:IFT(...)
	lualog('IFT:called')
	
	local args = { ... }
	for k, v in pairs(args) do
		select_meta[k].label = v
	end
	
	nextScript()
end

-- IF:選択肢表示 --
function commands:IFS()
	lualog('IFS:called')
	
	l = gcclayer:getChildByTag(LAYER_IF)
	l:setVisible(true)
end

-- CSK:シーンスキップ --
function commands:CSK( scene )
	scene_skip_no = tonumber( scene )
	nextScript()
end
